@@ -2,10 +2,11 @@ import hashlib
 import json
 from typing import Iterator, List, Tuple
 
-import cognite.air_ds_util.utils as utils
 from cognite.client.data_classes import Asset
 from cognite.experimental import CogniteClient
 
+from cognite.airworkflow.util.cdf import data_set_id
+
 from .scheduling import (
     Schedule,
     create_schedule_instance,
@@ -17,6 +18,8 @@ from .scheduling import (
 )
 
 AIR_INFRA = "airInfra"
+MODEL_POSTFIX = ":latest"
+DATA_SET_NAME = "AIR"
 
 
 def retrieve_infra_model_assets(client) -> Iterator[Asset]:
@@ -26,7 +29,7 @@ def retrieve_infra_model_assets(client) -> Iterator[Asset]:
 
 
 def retrieve_schedule_asset_without_asset(client: CogniteClient) -> List:
-    air_assets: List[Asset] = client.assets.list(data_set_ids=[utils.retrieve_data_set_id(client)], limit=-1)
+    air_assets: List[Asset] = client.assets.list(data_set_ids=[data_set_id(client, DATA_SET_NAME)], limit=-1)
     assets: Iterator[Asset] = retrieve_infra_model_assets(client)
     assets_to_create: List[Asset] = []
     for asset in assets:
@@ -39,13 +42,14 @@ def create_asset(client, asset: Asset) -> str:
     data = json.dumps({})
     hashed = hashlib.md5(f"{asset.external_id}-{data}".encode()).hexdigest()  # nosec
     asset_to_create = Asset(
-        data_set_id=utils.retrieve_data_set_id(client),
+        data_set_id=data_set_id(client, DATA_SET_NAME),
         name=f"{asset.name} schedule",
         external_id=hashed,
         parent_external_id=asset.external_id,
         metadata={"data": data},
     )
-    client.assets.create(asset_to_create)
+    if client.assets.retrieve(external_id=hashed) is None:
+        client.assets.create(asset_to_create)
     return hashed
 
 
@@ -54,7 +58,7 @@ def retrieve_undeployed_infra_schedule_assets(
 ) -> List[Schedule]:
     schedule_assets = retrieve_schedule_assets(client)
     schedules = client.functions.schedules.list()
-    schedules_external_ids = map(lambda s: s.schedule_asset_external_id, schedules)
+    schedules_external_ids = map(lambda s: s.description, schedules)
     undeployed_schedule_assets = [i for i in schedule_assets if i.external_id not in schedules_external_ids]
     infra_schedule_assets = list(retrieve_infra_model_assets(client))
     undeployed_infra = filter(
@@ -95,18 +99,23 @@ def delete_double_schedules(client: CogniteClient, model_names: List):
             [client.functions.schedules.delete(j.id) for j in delete_ids]
 
 
+def functions_exist_and_ready_external_ids(client: CogniteClient) -> List[str]:
+    functions = client.functions.list()
+    ready_air_functions = [f.external_id for f in functions if f.owner == "AIR" and f.status == "Ready"]
+    return ready_air_functions
+
+
 def execute(client: CogniteClient):
     missing_schedule_assets = retrieve_schedule_asset_without_asset(client)
     [create_asset(client, asset) for asset in missing_schedule_assets]
     to_be_deployed = retrieve_undeployed_infra_schedule_assets(client)
+    existing_air_functions = functions_exist_and_ready_external_ids(client)
     # check if function exists
-    existing_deployed = [
-        model for model in to_be_deployed if client.functions.retrieve(external_id=model.function_external_id)
-    ]
+    existing_deployed = [model for model in to_be_deployed if model.model + MODEL_POSTFIX in existing_air_functions]
     if len(existing_deployed):
         create_schedules(client, existing_deployed)
     updated, to_be_deleted = retrieve_updated_infra_schedules(client)
-    existing_updated = [model for model in updated if client.functions.retrieve(external_id=model.function_external_id)]
+    existing_updated = [model for model in updated if model.model + MODEL_POSTFIX in existing_air_functions]
     if len(existing_updated):
         update_schedules(client, existing_updated, to_be_deleted)
     print(f"Created: {len(existing_deployed)}, Updated: {len(existing_updated)}")
